\ 9850.fs -- 9850 DDS Driver for single DDS -- 20Feb14 rjn
\ 
] here ( *start)
\
\ 
0 [if] \ ------------------------[ DDS I/O ]-----------------------------------
\
\ these are defined in init.fs, shown here for code readability
\ 
:m wclk        [ 0 .P0 ] m;
:m frud        [ 1 .P0 ] m;
:m dds-data    [ 2 .P0 ] m;
:m mrst        [ 3 .P0 ] m;
\ 
[then] \ ----------------------------------------------------------------------
\ 
-: fload     [ frud set ]  nop [ frud clr  ] ;  \ assumes FRUD is low
:m clock     [ wclk set ]  nop  [ wclk clr ] m;  \ 9850 clocks low to high
\  
0 [if] \ --------------------[ chip initialization ]---------------------------
\  
1. Reset must remain high for at least 5 sysclk (500 us for 10 kHz sysclk)
2. Minimum reset recovery is 2 sysclk
3. Reset output latency is 13 clock cycles from start of reset
4. After reset, chip is in parallel mode.  Serial mode must be set after a
   reset.
\ 
[then] \ ----------------------------------------------------------------------

0 [if] \ ------- old reset
\ 
\ reset must remain high for at least 5 sysclk
\ minimum reset recovery is 2 sysclk
-: us  ( n - )   \ 100 # |us =~ 115 us
   7 #for  
      6 #  6 #for nop 6 #next  
   7 #next ;
-: 500us  500 # us ;

\ reset DDS -- should be good for clock rates down to 10 kHz sysclk
: /dds  500us  [ mrst set ] 500us [ mrst clr ]  500us 500us 500us ;
\ 
[then] \ ----------------------------------------------------------------------

-: 1us  1 # us  ;

\ reset DDS -- good down to at least ~17 MHz
\ a 5 sysclk reset width for 17 MHz is (5)(60ns)=300ns
\ for convenience, use 1us which is lower than 17 MHz  
-: /dds  ( 500us) nop nop  [ mrst set ] 1us [ mrst clr ]  1us ;
\ 
0 [if] \ -----[ init chip ]
\ reset must remain high for at least 5 clocks
: many nop nop nop nop nop nop nop nop ;
: /9850 
   [ mrst clr  wclk clr  frud clr ]
   [ mrst set ] many [ mrst clr ] ;
\ 
\ -----[ init serial mode ]
\ per figs 10 & 11 on rev H datasheet
\ assumes D0 high, D1 high, D2 low
: sinit   /9850  fload  clock-a fload ;
\
[then] \ ----------------------------------------------------------------------
\ 
\ -----[ init serial mode ]
\ per figs 10 & 11 on rev H datasheet
\ assumes D0 high, D1 high, D2 low
-: /smode  /dds  ( fload)  clock  fload ;
\
\ -----[ send a byte to DDS A ]
-: sbyte  ( n -)  \ send least significant bit first
   [ wclk clr ]  ( wclk should be clear, but ...)
   8 # 7 #for
      [ rrc dds-data movcb ]      \ send lsb from T to dds-data
      nop
      clock  \ 9850 clocks low to high  
   7 #next   drop ;
\ 
\ :m 0byte  0 #  sbyte m;  \ send 0 byte to DDS A
\ 
\ : sbyte-test  begin $aa sbyte ie-switch 0=until. ;
\ 
[  \ --- gforth definitions!
\ 
0 [if] \ -------------[ frequency calculation notes ]--------------------------
\
1. For a 100 MHz crystal:  (2^32)/100,000,000 = 42.94967295 counts per hertz
                                              = $19999999 
2. For a 125 MHz crystal:  (2^32)/125,000,000 = 34.35973836 counts per hertz 
                                              = $cccccccc   counts per hertz
[then] \ ----------------------------------------------------------------------
\ 
	4294967295 constant 2^32
    16000000 constant freq  \ crystal frequency (Dev Board)
\ 24577966/268435456 = 0.0915600583 Hz/count   
\	25000000 constant freq
\ 125000000/4294967295 = 0.0291038304 Hz/count
	: U*/MOD  >R UM* R> UM/MOD ;
	: U*/  U*/MOD NIP ;
	: (hz)  ( n1 - n2)   2^32 freq U*/ ;  
	: hz  ( freq -)   (hz) hex . decimal ;	 
\  
\ convert a gforth number (32 bits) to two 16-bit "doubles" for MyForth
: >doubles  ( n - leastn mostn)   
   dup $ffff0000 and  16 rshift swap $0000ffff and  swap ;
: n>bytes  ( n -- n1 n2)   dup $ff00 and  8 rshift  swap  $00ff and ;
: >singles  ( n - n1 n2 n3 n4)   \ convert to four MyForth singles (bytes)
   >doubles swap n>bytes  rot  n>bytes   ;    \ usage: 843 (hz) >singles

: hz+  ( o n - n1 n2)   (hz) + >doubles ;  \ usage: 7 261 hz+ ## ## tone 
: tbytes  ( n - n1 n2 n3 n4)   (hz) >singles ;   \ tone bytes
: hertz  ( n - n1 n2)   (hz) >doubles ;  \ usage: 20000 hertz ## ## tone 
\ 
] \ --- back to MyForth

-: ~phase  ( n - n')   $1f # and  2* 2* 2*  ;  \ shift phase bytes to upper
\ -: ~phase  ( n - n')  \ version for 9850/9851   
\    $1f # and  2* 2* 2*   \ shift phase bytes to upper
\   9851? #@ 0=if drop ; then drop $01 # ior ;
\ 
-: sbytes  ( msb .. lsb)  sbyte sbyte sbyte sbyte ; \ lsb sent first
\
\ -: 5zeroes  0 ##  0 ##  0 # ;
\ -: 0send  5zeroes sbytes sbyte ;  

-: 0sbyte  0 # sbyte ;
-: (send)  ( msb n2 n3 lsb -)   sbytes  0sbyte ;
-: send    (send) fload  ;

-: !abytes   afreq # @quad  (send) ;

-: (zz)   -int  !abytes  fload +int ;
\ 
\ -----[ a -- hold=>a, output to a]
\ 
:m (A)   a-fload (zz) m;

1 [if] \ -------------------[ test ]-------------------------------------------
\ 
: 1000hz   /smode [ 1000 hertz ] ## swap ## swap ( msb .. lsb) send  ;
: 2000hz   /smode  \ 125 MHz clock 
   $6f # sbyte  $0c # sbyte  $01 # sbyte  $00 # sbyte  0sbyte  fload  ;
\ 
[then] \ ----------------------------------------------------------------------
\ 
here ( *stop) [ swap -  cr .( 9850.fs            ) . .( bytes) ]
\
\
0 [if] ---------------------[ Revision History ]-------------------------------

Date	  By  Description
======= === ===================================================================
20Feb14 rjn cloned from s-9850.fs for dstro app.  Changed for single DDS
03Apr13 rjn cloned from dcon-9850.fs
15Jan13 rjn cloned from serf
21Sep12 rjn changed for two W_CLK lines (e.g., added b-sbyte)
31Aug12 rjn misc. cleanup
10Aug12 rjn added notes
09Aug12 rjn more changes for dds app.
26Jul12 rjn minor changes for DDS application (I/O now defined in init-dds.fs)
04Feb11 rjn initial version

[then] \ ----------------------------------------------------------------------
