\ dstro_timer0.fs -- strobe timer 0 interrupt -- 21Apr14 rjn
\
\
] here ( *start)
\
\ -----[ interrupt enable/disable ] 
:m |disable-interrupts  [ 7 .IE clr ] m;
:m |enable-interrupts   [ 7 .IE set ] m;
\ -: enable-interrupts   |enable-interrupts ;
\ -: disable-interrupts  |disable-interrupts ;
\ :m -int  |disable-interrupts m;  \ now in bootloader
\ :m +int  |enable-interrupts m;

:m |start-T0  
	[ 4 .TCON set ]   \ start timer 0   
	[ 1 .IE   set ]   \ enable overflow interrupt 
m;

-: start-T0  |start-T0 ;

:m |stop-T0   [ 4 .TCON clr  ( 5 .TCON clr) ]  m;
-: stop-T0   |stop-T0 ;

:m save-state  \ ~ 80 ns per instruction, ~ 960 ns total
   [ t push  psw push  $82 push ( DPL)  DPH push  
     0 push  1 push  2 push  3 push  4 push  5 push  6 push  7 push ] m; 

:m restore-state  \ ~ 80 ns per instruction
   [ 7 pop  6 pop  5 pop  4 pop  3 pop  2 pop  1 pop  0 pop  
     DPH pop  $82 pop  psw pop  t pop ] m;
\
1 [if] \ --------------------------[ 1 ms tick timers ]-------------------------
\ 
\ note -- used for LCD and frequency updates
\ 
cpuHERE constant ticks0 1 cpuALLOT
\ 
:m |ticks0-  [ ticks0 dec ] m;
:m !ticks0  ( n -)  -int ticks0 #! +int m;  \ max is $ff (~256 ms)
:m @ticks0  ( - n)  -int ticks0 #@ +int m;
\ 
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ --------------[ second tick timer ]-----------------------------------
\ 
cpuHERE constant ticks1 1 cpuALLOT
\ 
:m |ticks1-  [ ticks1 dec ] m;
:m !ticks1  ( n -)  -int ticks1 #! +int m;  \ max is $ff (~256 ms)
:m @ticks1  ( - n)  -int ticks1 #@ +int m; 
\
[then] \ ----------------------------------------------------------------------
\ 
\ 
0 [if] \ --------------------[ ISR Setup & Timing ]----------------------------

1. timer0 is set up as 16-bit in bootloader

2. With timer0 set at 2 ms, with save/restore state:

	+ 7.2 us to service interrupt, no IE activity
	+ 9.3 us to service interrupt, with IE activity

[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ -----------------[ edge detection ]-----------------------------------
\ 

cpuHere constant prev-cstate 1 cpuAllot  \ $ff if previous cmp state was high

-: do-pedge   \ actions for low to high edge transition
   [ 4 .P0 toggle ] ;
-: do-nedge   \ actions for low to high edge transition
   [ 4 .P0 toggle ] ;
   
-: current-cstate  ( - ?)   CPT0CN #@ 2* -if drop $ff # ; then drop 0 # ;   
-: ?pos-edge  \ look for cmp0 positive edge ( low to high transition)
   prev-cstate #@ current-cstate xor 0=if drop ( same)  ; then drop
\   current-cstate dup prev-cstate #!  -if drop do-edge  ; then drop ;
   current-cstate dup prev-cstate #!  -if drop do-pedge  ; then drop do-nedge ;
\ 
[then] \ ----------------------------------------------------------------------
\ 
\ Note: isr takes ~2.5 usec to update the timer count
\ resting rate is ~4 usec
TIM0 interrupt : timer0-isr
	|disable-interrupts
   |stop-T0
\   [ 4 .P0 set ]  \ for timing
   save-state         \ ~960 ns
   |ticks0-           \ 1 ms tick timer
   |?ie-update
   restore-state      \ ~960 ns
	$F7 # TH0 #!  $FF # TL0 #!  \ reload timer, ~  1 ms
\	$FF # TH0 #!  $40 # TL0 #!  \ reload timer, ~  100 us
\   [ 4 .P0 clr ]  \ for timing
   |start-T0        
   |enable-interrupts
   reti
\
-: /timer0  -int  |start-T0  +int ;

0 [if] \ ------------------------- FYI ]---------------------------------------
\	$FF # TH0 #!  $40 # TL0 #!  \ reload timer, ~100 us
\	$FF # TH0 #!  $09 # TL0 #!  \ reload timer, ~125 us
\	$FE # TH0 #!  $0A # TL0 #!  \ reload timer, ~250 us
\	$F8 # TH0 #!  $FF # TL0 #!  \ reload timer, ~884 us
\	$F0 # TH0 #!  $09 # TL0 #!  \ reload timer, ~  2 ms
[then] \ ----------------------------------------------------------------------		
\ 
here ( *stop) [ swap -  cr .( dstro_timer0.fs    ) . .( bytes) ]
\ 
\ 
0 [if] ---------------------[ Revision History ]-------------------------------

Date	  By  Description
======= === ===================================================================
21Apr14 rjn changed name prior to production release
02Apr13 rjn minor changes for send
23Mar11 rjn adapted timer0.fs in WARB4 for XLATE

------------------------------------------------------------------------ [then]

