\ freqs.fs -- calculate & load DDS frequency values -- 03Mar14 rjn
\ 
]  here  ( *start)
\ 
\ -----[ hex/decimal flag ]
\ 
cpuHere constant hex? 1 cpuAllot  \ false=decimal in hold, true=hex in hold
:m !hex   $ff # hex? #!  m;
:m !dec     0 # hex? #!  m;
\ 
-: x>hold   ( q - )   !hold  !hex ;
\ 
\ ----- 16 MHz clock = (2**32)/(16,000,000) = 268.435456 counts/Hertz
here constant 16MHz-scale    \ value here, lsb first
\ $82 , $de , $1b , $43 ,    \ 2**22 scaling (~6 decimal places)
\ $06 , $bd , $37 , $86 ,    \ 2**23 scaling, 1 Hertz
\ $4d , $f9 , $6b , $0d ,      \ 2**23 scaling, 0.1 Hertz
$dc , $31 , $af , $02 ,      \ 2**24 scaling, 0.01 Hertz

-: read-d16  ( - q)  16MHz-scale ##  @qflash ;
: x16   read-d16  x>hold ;
\ 
0 [if] \ -------------------[ Misc. Scaling ]----------------------------------
\ 
\ ----- 25 MHz clock (171.79869184 counts/Hertz)
here constant 25MHz-scale  \ value here, lsb first
$11 , $77 , $cc , $ab ,    \ 2**24 scaling (~7 decimal places)
-: read-d25   25MHz-scale ##  @qflash ;
: x25   read-d25  x>hold ;
\ 
\ ----- 50 MHz clock (85.89934592 counts/Hertz)
here constant 50MHz-scale  \ value here, lsb first
$88 , $3b , $e6 , $55 ,    \ 2**24 scaling (~7 decimal places)
\ $e2 , $8e , $79 , $15 ,    \ 2**22 scaling (~6 decimal places)
-: read-d50   50MHz-scale ##  @qflash ;
: x50   read-d50  x>hold ;
\ 
\ ----- 100 MHz clock (42.94967296 counts/Hertz)
here constant 100MHz-scale  \ value here, lsb first
\ $71 , $c7 , $bc , $0a ,     \ 2**22 scaling
$c4 , $1d , $f3 , $2a ,     \ 2**24 scaling
-: read-d100   100MHz-scale ##  @qflash ;
: x100   read-d100  x>hold ;
\ 
\ ----- 125 MHz clock (34.35973836 counts/Hertz)
\ 
here constant 125MHz-scale  \ value here, lsb first
$d0 , $17 , $5c , $22 ,     \ 2**24 scaling
\ $f4 , $05 , $97 , $08 ,     \ 2**22 scaling
-: read-d125  ( - q)   125MHz-scale ##  @qflash ;
: x125   read-d125  x>hold ;
\  
\ ----- 180 MHz clock (34.35973836 counts/Hertz)
here constant 180MHz-scale  \ scale value goes here, lsb first
$de , $65 , $dc , $17 ,     \ 2**24
\ $77 , $19 , $f7 , $05 ,     \ 2**22
-: read-d180   180MHz-scale ##  @qflash ;
: x180   read-d180  x>hold ;
\ 
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ ------------------[ HOW IT WORKS ]------------------------------------
\
1. To translate a DDS load value into a frequency that can be displayed on
   the LCD, the current DDS load frequency in the incremental encoder
   rugister must be multiplied by the scaling factor.
2. This multiplication is done by loading the DDS load bytes into the
   freq register (dq), loading the scaled "counts per Hertz" value in tho 
   scale (q) register and executing multiply.  The scaled results are in 
   the prod register (dq).
3. The scaling factor is loaded into scale by load-ascale.  
   The ascale register (q) is an artifact from DCON where there are both
   A and B DDSes.  The word hold>ascale loads the ascale register from the
   holding register.  The ascale register can also be set once at startup
   if the scaling factor will not change (e.g., a single DDS application).
4. The scale factor is looked up and loaded into the hold register by words
   such as x125 for a 125 MHz crystal.  Then, hold is loaded into ascale by
   hold>ascale.  This is the normal way to load ascale: x125 hold>ascale .
   This sequence is performed by /freqs.
5. To translate the scaled multiplication result (in prod) to a frequency 
   that can be displayed on the LCD, it must be "de-scaled' by the 
   scaling factor (currently 2**24).  This de-scaling is done within the
   multiplication "doit" word, mscale.  Within mscale, the actual
   de-scaling (a bunch of divide by twos) is done by de-scale.
6. The LCD display routines use the contents of the holding register, hold (q).
   Thus, the de-scaled multiplication results (in prod) must be transfered to
   hold (q) before the results can be displayed.
7. The de-scaled results of a multiplication are first transfered to the
   afreq (q) register for convenient loading into hold (q) for display.
   Again, afreq is an artifact of an application with both an A and a B DDS.
   The word that transfers prod to afreq is prod>afreq .
8. The word a-fload performs the entire operation of loading the scale factor
   from ascale, transfering the holding register to the frequency register
   performing the multiplication, descaling the result and transfering the
   results to afreq (q).
\ 
[then] \ ----------------------------------------------------------------------
\ 
\ :m de-scale  24 # 4 #for prod/2' 4 #next m; \ de-scale by 2**24
:m de-scale  24 # 4 #for prod/2' 4 #next m; \ de-scale by 2**23
\ :m de-scale  22 # 4 #for prod/2' 4 #next m; \ de-scale by 2**22
\ 
-: hold>ascale  @hold<  ascale # !quad ;
-: hold>freq  0 ## 0 ## freq # !quad  @hold< freq4 # !quad ;
-: mscale  hold>freq multiply de-scale ;
\ 
\ -----[ register fetch & store utilities ]
\ 

-: @prod-lsb<  ( - q)   prod4 # @quad< ;
\ 
-: prod>afreq  @prod-lsb<  afreq # !quad ;
:m load-ascale   ascale # @quad<  scale # !quad m;  
:m hold>afreq  @hold<  afreq # !quad m;  
:m afreq>hold  afreq # @quad<  !hold m;
-: a-fload  
   hex? #@ 0=if drop load-ascale mscale prod>afreq ; then drop hold>afreq ;
\ 
-: /freqs   ( x125) x16 hold>ascale  ;
\  
0 [if] \ -------------------------- test --------------------------------------
\ 
\ Example for 50 MHz frequency, 125 MHz xtal, scale scaled by 2**22:
\ 
\ ($2FAF080)($89705F4)=$199999995FBA00
\ dividing by 2**22=$66666665=1,717,986,917 (vs. 1,717,986,918)
\ 
: !50mhz-freq   $80 # $F0 # $FA # $02 # 0 ##  0 ##  !freq ; \ $2FAF080 = 50 MHz
: rjn   !50mhz-freq 125mhz-fload .prod ;
\
[then] \ ----------------------------------------------------------------------
\ 
here ( * stop)  [ swap -  cr .( freqs.fs           ) . .( bytes) ]
\ 
0 [if] \ ------------------------[ revisions ]---------------------------------
\ 
Date	  By  Comment
======= === ===================================================================
03Mar14 rjn changed for 16 MHz clock and 2**23 scaling
25Feb14 rjn modified for dstro application (no b side stuff)
21Apr13 rjn changed flash references to use @qflash
09Apr13 rjn changed hold references (e.g., now using @hold<).
20Jan13 rjn went back to 2**24 scaling (good down to ~ 17 MHz), added x25 
19Jan13 rjn added additional scaling factors (e.g., x50)
15Jan13 rjn cloned from serf
04Sep12 rjn misc. changes to reduce code size
29Aug12 rjn improved factoring, played with A,B synchronization
16Aug12 rjn initial version
\ 
[then] \ ----------------------------------------------------------------------
