\ init.fs -- I/O initialization for dstro -- 04Mar14 rjn
\
]  here  \ start marker
\ 
0 [if] ---------------------------[ Notes ]------------------------------------

1. Configures 850 chip on toolstick board.
2. To be called as part of cold so that after a reset the hardware 
    returns to these settings
3. If a bit needs changing in the application, "and/orl" it.
4. 850 I/O assignments for MINT Board:  

         MINT
   Port	Pin Type Function
   ----	--- ---- --------------------------------------------------------------
   P0.0	  7   PP DDS wclk
   P0.1	  5   PP DDS frud (FQ_UD)
   P0.2    3   PP DDS data
   P0.3	  4   PP DDS mrst 
   P0.4	  6   PP reserved for serial output or for on-board LED1
                  For dstro, the cpt0 output should appear here
   P0.5	  8   PP reserved for serial input
   P0.6   10   AI comparator positive input 
   P0.7	 12   PP comparator output
   P1.0	 14   PP LCD DB4
   P1.1	 16   PP LCD DB5
   P1.2   18   PP LCD DB6
   P1.3	 20   PP LCD DB7
   P1.4	 22   DI IE B
   P1.5	 21   PP IE A
   P1.6   19   PP LCD E
   P1.7   17   PP LCD RS
   P2.0   15   BD unassigned
   P2.1   13   PP STROBE LED (FET DRIVEN)
        2,9,24 GND
           1   VDD (5 Volts)
          13   RST/C2CK (to prog debug adapter) 
\ 
[then] \ ----------------------------------------------------------------------
\ 
\ -----[ Port 0 ]
\ 
:m p00     [ 0 .P0 ] m;  \ unassigned
:m p01     [ 1 .P0 ] m;  \ unassigned
:m p02     [ 2 .P0 ] m;  \ IE switch
:m p03     [ 3 .P0 ] m;  \ external clock
:m p04     [ 4 .P0 ] m;  \ unassigned
:m p05     [ 5 .P0 ] m;  \ unassigned
:m p06     [ 6 .P0 ] m;  \ IEB
:m p07     [ 7 .P0 ] m;  \ IEA
\ 
\ -----[ Port 1 ]
\ 
:m p10     [ 0 .P1 ] m;  \ DB4
:m p11     [ 1 .P1 ] m;  \ DB5
:m p12     [ 2 .P1 ] m;  \ DB6
:m p13     [ 3 .P1 ] m;  \ DB7
:m p14     [ 4 .P1 ] m;  \ not assigned
:m p15     [ 5 .P1 ] m;  \ not assigned
:m p16     [ 6 .P1 ] m;  \ E
:m p17     [ 7 .P1 ] m;  \ RS
\ 
\ -----[ Port 2 ]
\ 
:m p20     [ 0 .P2 ] m;  \ 
:m p21     [ 1 .P2 ] m;  \ 
\ 
1 [if] \ ------------------------[ LED ]---------------------------------------

:m led  p21 m;
:m led-on  [ p21 set ] m;
:m led-off [ p21 clr ] m;
\ 
[then] \ ----------------------------------------------------------------------
\ 
1 [if] \ ------------------------[ DDS ]---------------------------------------
\ 
:m wclk        [ 0 .P0 ] m;
:m frud        [ 1 .P0 ] m;
:m dds-data    [ 2 .P0 ] m;
:m mrst        [ 3 .P0 ] m;
\ 
[then] \ ----------------------------------------------------------------------
\ 
1 [if] \ ---------------------[ misc. I/O ]---------------------------------------

:m mint_led  [ 4 .P0 ] m;
\ 
[then] \ ----------------------------------------------------------------------
\ 
1 [if] \ --------------------------[ IE ]--------------------------------------
\ 
:m ie-switch   [ 0 .P2 ] m;  \ encoder switch
:m iea         [ 5 .P1 ] m;  \ encoder phase a
:m ieb         [ 4 .P1 ] m;
\ 
\ switch reading, debounce, etc.
\ 
\ :m 80ms-delay  $0a # delay765 m;  \ debounce delay
-: 80ms-delay  80 # ms ;
\ : test-80ms  begin 80ms-delay ~cbit again
\ 
\ --- continue on switch up and down
-: ?ie-up    begin 80ms-delay ie-switch until.   ;  \ continue if ie-switch is up
-: ?ie-down  begin 80ms-delay ie-switch 0=until. ;  \ continue if ie-switch is down
-: ?switch-up  ?ie-up 80ms-delay ;  \ with debounce
\ 
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ ------------------------[ LCD I/O ]-----------------------------------
\ 
\ :m push-pull  $ff # P1MDOUT ior! m;  \ set all outputs as push-pull
:m pins  P1 m;
:m dirs  P1MDOUT m;

:m .e       [ 6 .P1 ] m;  
:m .rs      [ 7 .P1 ] m;    

\ these are defined in lcd16.fs -- must be there -- name conflict?
\ -: instruction  [ .rs clr ] ;
\ -: data         [ .rs set ] ;
\ 
[then] \ ----------------------------------------------------------------------
\ 
\ 
0 [if] \ -------------------[ I/O initialization ]-----------------------------

 1. Steps for port I/O initialization, per datasheet:
	Step 1 - Select input mode (analog or digital) for all port pins
		 using PnMDIN.  If the pin is in analog mode, a "1" must also
		 be written to the corresponding Port Latch.  Default for
		 PnMDIN is $ff (pins not configured as analog inputs).
	Step 2 - Select the output mode using PnMDOUT.  
		  Output Modes - 0=open drain, 1=push pull
	Step 3 - Assign port pins to desired peripherals using XBRn registers.
	Step 4 - Enable the crossbar (XBARE="1")
2. Pins used as comparator or ADC inputs should be configured as analog inputs.
3. Analog input pins should be skipped by the crossbar.
4. All analog pins must have a "1" set in the corresponding Port Latch reg.
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ ----------------------[ PCA ]-----------------------------------------
\ 
1. PCA0CPH0 and PCA0CPL0 control duty cycle
2. For synchronous operation, set ECCF0 and MAT0 & check PCACCF0 flag 
3. For 16 bit PWM, SYSCLK source, freq.= 373.8 Hz (47K/10uf filter)
4. 8 bit PWM:
     + load value in PCACPH0
     + freq. = 23.9 kHz, SYSCLK/4
     + freq. = 95.7 kHz, SYSCLK -- see table below

     load  positive   negative   Voltage
             (usec)    (usec)
     ====  ========   ========   ========
     $ff                         14 mV
     $f0   0.7        9.8        0.204
     $e0   1.3        9.1        0.406
     $d0                         0.608
     $c0   2.6        7.9        0.81
     $af   3.4        7.1        1.026       (3.4/10.5)(3.256) = 1.03
     $a0   4.0        6.5        1.21
     $90                         1.42
     $80   5.3        5.2        1.62
     $7d                         1.66
     $70                         1.82
     $60   6.5        3.9        2.02
     $50                         2.23
     $40   7.8        2.6        2.43
     $30                         2.63
     $1b                         2.90
     $10   9.8        0.6        3.04
     $01                         3.22
     $00                         3.24

5.  Simple PWM setting and readout:

   -: go  /app  0lcd lcd-coff
      begin
        @option  dup !pwm  line1  h.lcd  25 # ms 
      again
\ 
[then] \ ----------------------------------------------------------------------
\ 
:m //pca
   $08 # PCA0MD #!      \ PCA0 clock source is SYSCLK
   $00 # PCA0CN #!      \ set bit 6 to run PCA0 (init to stop)
\   $c2 # PCA0CPM0 #!    \ PWM enabled (bit 1), cmp0 function enable (ECOM bit 6), 
                        \ 16 bit PWM (bit 7)
    $42 # PCA0CPM0 #!    \ no 16-bit PWM
\   $00 # PCA0PWM #!     \ default  -- 8 bits if no 16 bit PWM
                         \ 8-11th bit overflotw int. disabled
\   $00 # PCA0POL #!     \ default (normal polarity)
\   $00 # PCA0CLR #!     \ default
m;

\ 
:m +pca0    [ 6 .PCA0CN set ] m;
:m -pca0    [ 6 .PCA0CN clr ] m;
:m normal-polarity  $00 # PCA0POL #! m;
:m invert-polarity  $01 # PCA0POL #! m;
\ 
\ ----- 16-bit
\ -: !pwm  ( d -)   push PCA0CPL0 #! pop PCA0CPH0 #! ;
\ -: /pwm   $8fff ## !pwm  +pca0 ;
\ 
\ ----- 8-bit
-: !pwm  ( n -)   PCA0CPH0 #! ;
-: /pwm   $b0 # !pwm  ( invert-polarity) +pca0 ;


-: /chip 
   $00 # XBR2 #!        \ disable xbr
\ 
\ -----[ configure I/O ] 
\ 
\ ----- port 0
   $6f # P0SKIP #!      \ skip P0[0-3], P0[5,6] 
                        \ cmp0- on P0.5 cmp0+ on P0.6, cmp0 out on P0.4
   $56 # CPT0MX #!      \ mux: cmp0= to P0.5, cmp0+ to P0.6
   $9f # P0MDIN #!      \ P0[5,6] analog input
   $9f # P0MDOUT #!     \ P0[0,1,2,3,4] PP, P0[5,6] DI, P0.7 PP
\   $8f # P0MDOUT #!     \ P0[0-3] and P0.7 are outputs
   $80 # CPT0CN  #!     \ enable cmp0, no hysteresis
                        \ read bit 6 to get output state
   $00 # CPT0MD #!      \ cpt0 fast, high pwr (100 ns response)
\ 
\ ----- port 1
   $cf # P1MDOUT #!     \ P1[0,1,2,3,6,7] PP, P1[4,5] DI
\ 
\ ----- port2   
   $02 # P2MDOUT #!     \ P2.1 is PP for strobe LED, P2.0 is switch input
\ 
   $00 # CLKSEL  #!     \ 24.5 MHz internal clock
   $00 # CLKSEL  #!     \ internal clock, no SYSCLK scaling
\   $07 # PRTDRV #!      \ enable all as high drive (default)
   //pca
\ 
\ ----- crossbars
\ 
   $08 # XBR0 #!        \ enable cmp0 output, on P0.4
\   $00 # XBR0 #!        \ no resources routed to port pins
   $01 # XBR1 #!        \ CEX0 routed to port pin (for PWM), on P0.7
\   $00 # XBR1 #!        \ no resources routed to port pins
   $40 # XBR2 #!        \ enable XBR, weak pullups enabled
\ 
\ -----[ init application I/O ]
\ 
   [  ( IE)  iea  set  ieb  set  ie-switch set  mint_led set ( off)
      ( DDS) mrst clr  wclk clr  frud clr ] ;
\ 
\ 
\ 
here [ swap -  ( -- n)
cs? [if] cr .( init.fs            ) . .( bytes) [else] drop [then] 
]

\
\
0 [if] ---------------------[ Revision History ]-------------------------------

Date	  By  Description
======= === ===================================================================
04Mar14 rjn configured P0.6 as cpt0 input, P0.4 as cpt0 output
23Feb14 rjn configuration for dstro application
17Aug13 rjn changed for 850 chip
10aug13 rjn changed for conditional compilation summary
09Feb13 rjn cloned from init-dcon.fs
\ 
[then] \ ----------------------------------------------------------------------

