\ intervals.fs -- interval translation for menus -- 17Apr13 rjn
\ 
\
] here ( *start)
\
0 [if] -------------[ Equally Spaced Menu Intervals ]--------------------------

1. The routines below were designed to approximate evenly spaced intervals
   between menu items.
2. Menu items are displayed based on the current least significant byte of
   the four-byte incremental encoder (IE) accumulator.  Thus, for a given
   number of menu items, it is desirable to evenly space the items based on
   an IE range of 256 counts.
3. One turn of the IE is about 64 counts.  Thus, a full count of 255 represents
   four turns of the IE or about 1440 degrees.
4. The menu spacing in degrees is 1440 divided by the number of intervals in
   the 256 count total.  Thus, 13 intervals yields 1440/13 =~ 110 degrees.
   Similarly, intervals of 10 and 12 give 144 and 120, respectively.
5. Because the difference between 110 and 144 degrees is only marginally 
   noticable to the user, all menus item changes will appear to be evenly 
   spaced at about 120 degrees (1/3 of a knob rotation).
6. Of course, it is not possible to exactly evenly space intervals of 10, 12
   or 13 withing a count of 256 (these are not powers of 2).  But, the
   words below give an approximate even spacing although some items are spaced
   at larger or smaller increments than others.  When brute force range
   comparison is used, it is possible to distribute the excess counts more or
   less evenly over the total range.  However, brute force comparison takes
   much more code than a looping definition (e.g., 129 bytes vs. 32).  Even 
   so, the interval difference will not be noticed by the user.  In the code
   below, the interval discrepancies are noted and are on the order of 4
   counts out of 26.
7. A total number of menu items of 5, 3 or 4 can be accommodated by just 
   "modding" the results to wrap the counts around.  For example, a 3-item
   menu can be implemented by converting the raw count from the encoder to
   a range of 0 to 11 (in equally spaced increments) and then doing a mod
   3 calculation on the result.  Similarly, a range of 0-9 can be used with
   5 menu items by doing a mod 5 calculation.
8. For some menu items, such as 13, it is necessary to do a brute force
   "even" distribution of counts over the range.  Hence the code in the
   Word 256>13 below.
9. Menu items of 2 and 6 can use the "mod" technique.  Seven, nine and 
   eleven items are cases that are not yet needed, but can be coded to give
   acceptable results using the examples given below. 
10. Note that the looping versions equally space all items except one.
    Thus, all of the spacing error accumulates in a single case instead
    of being distributed throughout the range in smaller steps as can
    be coded using the brute force distribution Words.
    For the one output digit that is the exception, it might have a span of
    more or fewer counts than the other digits, depending on the interval 
    size.  Thus, for 256>10, the output digit 9 spans a range of only 22 
    counts instead of 26.  Similarly, for 256>12, the digit 11 spans a range
    of 25 counts instead of 20.
------------------------------------------------------------------------ [then]
\ 
\  
:m 256>8   ( n - n' )    8 # |u/mod swap drop m;  \ convert 0-255 to 0-7

0 [if] 
\ ~129 bytes 
-: 256>10  ( n -)  \ translate 0-255 to 0-9
   dup  25 # <if drop drop  0 #   ; then drop \ 26 
   dup  51 # <if drop drop  1 #   ; then drop \ 26 
   dup  77 # <if drop drop  2 #   ; then drop  
   dup 103 # <if drop drop  3 #   ; then drop  
   dup 129 # <if drop drop  4 #   ; then drop
   dup 155 # <if drop drop  5 #   ; then drop \ 25
   dup 179 # <if drop drop  6 #   ; then drop
   dup 205 # <if drop drop  7 #   ; then drop
   dup 231 # <if drop drop  8 #   ; then drop
   drop                     9 #   ;           \ 24 
[then] 

-: 256>10  ( n - n')   \ ~32 bytes -- translate 0-255 to 0-9, steps of ~26 
\ Note: only 22 steps for 9; range for 8 is 0-3 & 234-255 (26)
   -1 #  swap dup
   begin 
      drop swap 1+ swap  26 # negate + dup 
   26 # <until drop drop ;

-: 256>6  ( n - n')   \ ~32 bytes -- translate 0-255 to 0-4, steps of ~42 
   -1 #  swap dup
   begin 
      drop swap 1+ swap  42 # negate + dup 
   26 # <until drop drop ;

0 [if] 
\ ~ 21 counts/change 
-: 256>12  ( n - n')   \ translate 0-255 into 0-12
   dup  21 # <if drop drop  0 #  ; then drop \ 22 
   dup  42 # <if drop drop  1 #  ; then drop \ 21 
   dup  63 # <if drop drop  2 #  ; then drop \ 21
   dup  84 # <if drop drop  3 #  ; then drop \ 21 
   dup 105 # <if drop drop  4 #  ; then drop \ 21
   dup 127 # <if drop drop  5 #  ; then drop \ 22
   dup 148 # <if drop drop  6 #  ; then drop \ 21
   dup 169 # <if drop drop  7 #  ; then drop \ 21
   dup 190 # <if drop drop  8 #  ; then drop \ 21
   dup 211 # <if drop drop  9 #  ; then drop \ 21
   dup 233 # <if drop drop 10 #  ; then drop \ 22
   drop                    11 #  ;           \ 22
\  
[then] \ ----------------------------------------------------------------------
\ 
\ Notes for looping version of 256>12:
\   Input Range   Result
\   -----------   ------
\        20-39      0
\        40-59      1
\        60-79      2
\        80-99      3
\       100-119     4
\       120-139     5
\       140-159     6
\       160-179     7
\       180-199     8
\       200-219     9
\       220-239    10 
\       240-255    11
\         0-9      11 (25 steps)
\ 
-: 256>12  ( n - n')   \ translate 0-255 to 0-11, steps of ~20
   -1 #  swap dup
   begin 
      drop swap 1+ swap  20 # negate + dup 
   20 # <until drop drop ;


\ ~ 19 counts/change, 197 bytes
-: 256>13  ( n - n')   \ translate 0-255 into 0-12
   dup  19 # <if drop drop  0 #  ; then drop \ 20 
   dup  38 # <if drop drop  1 #  ; then drop  
   dup  57 # <if drop drop  2 #  ; then drop 
   dup  76 # <if drop drop  3 #  ; then drop  
   dup  96 # <if drop drop  4 #  ; then drop
   dup 116 # <if drop drop  5 #  ; then drop
   dup 136 # <if drop drop  6 #  ; then drop
   dup 156 # <if drop drop  7 #  ; then drop
   dup 176 # <if drop drop  8 #  ; then drop
   dup 196 # <if drop drop  9 #  ; then drop
   dup 216 # <if drop drop 10 #  ; then drop
   dup 236 # <if drop drop 11 #  ; then drop
   drop                    12 #  ;

:m 256>15  ( n - n' )   16 # |u/mod swap drop m;  \ convert 0-255 to 0-15

\ 
\
\ -----[ menu options ]-----
\ 
\ these mostly to save code
\ -: /options  ( - n)   ( 0lcd) @option ;
-: mod-part  ( n -)   swap u/mod drop ;

-: 6-spaced  ( n - n')  @option 256>6 mod-part ;    \ n = #menu items

-: 10-spaced  ( n - n')  @option 256>10 mod-part ;    \ n = #menu items

-: 12-spaced  ( n - n')  @option 256>12 mod-part ;    \ n = #menu items

-: 13-items  ( n - n')   @option 256>13 ;  \ 13 items per menu

:m @1of2  ( - n)   2 #  6-spaced m;
:m @1of3  ( - n)   3 # 12-spaced m;
:m @1of4  ( - n)   4 # 12-spaced m;
:m @1of5  ( - n)   5 # 10-spaced m;
:m @1of6  ( - n)   6 # 12-spaced m;
:m @1of7  ( - n)   @option 37 # u/mod nip m; \ n = #menu items
:m @1of8  ( - n)   @option 32 # u/mod nip m;
:m @1of10 ( - n)   @option 256>10 m;

\ -: stest  lcd-coff  begin 200 # ms @1of2 0lcd h.lcd ie-switch 0=until. ;
\ 
0 [if] \ ------------------------[ misc. ]-------------------------------------
\ 
-: d/5  ( d - n)  \ divide double by 5
   0 # push begin 5 ## dnegate d+ -if drop drop pop ; then pop 1+ push again ; 

-: @1of100  ( - n)   \ translate ie to 0-99, step size is ~ 5
   @doption $01 # and d/5  \ 0 to 102
   dup 100 # =if drop drop 99 # ; then drop
   dup 101 # =if drop drop 99 # ; then drop
   dup 102 # =if drop drop  0 # ; then drop ; 

-: test-roll  begin 0lcd @1of100  u.lcd 50 # ms ie-switch 0=until. ;

-: test-intervals 
   -1 #  dup begin 
      drop cr 1+ dup  dup u.  256>10   space . 
   dup 255 # xor 0=until drop drop ;

: test1  
   begin
      0lcd @option 256>10 ( 5 # u/mod nip) h.lcd
      35 # ms
   ie-switch 0=until. ;
\ 
[then] \ ----------------------------------------------------------------------
\ 
here ( *stop) [ swap -  cr .( intervals.fs       ) . .( bytes) ]
\
\ 
0 [if] \ ------------------------[ revisions ]---------------------------------

Date	By  Comment
=======	=== ===================================================================
17Apr13 rjn changed names from x-options to @1ofx   
16Apr13 rjn changes for SEND menu     
02Apr13 rjn minor changes for swis, eliminated head defs. mostly
\ 
[then] \ ----------------------------------------------------------------------
