\ lcd16.fs -- 16-pin lcd driver for 850 -- 15Feb14 rjn
\  
] here ( *)
\   
0 [if] \ ----------------[ Nibble Mode LCD, 16-pin ]---------------------------

LCD Port/	MINT    	Description
Pin Connect	Pin
--- -------	-----	-------------------------------------
 1  GND   	N/A	Vss
 2  +5V  	N/A	Vdd
 3  V0            contrast
 4  P1.7    17    RS - Instruction Register Select
 5  GND		N/A	R/W - H=read L=write Registers
 6  P1.6    19    E - Enable
 7  ---		--	   DB0 (N/C)
 8  ---		--	   DB1 (N/C)
 9  ---		--	   DB2 (N/C)
10  ---		--	   DB3 (N/C)
11  P1.0	   14	   DB4
12  P1.1    16	   DB5
13  P1.2    18	   DB6
14  P1.3    20    DB7
15  +5V		--    LED A through 150 Ohm resistor (backlight)
16  GND     --    LED K ground so LED is always on

[then] \ ----------------------------------------------------------------------
\ 
\ 
0 [if] \ ---------------[ FYI Delays from 25_delays.fs ]-----------------------

\ only valid for ~25 MHz sysclk

\ 100 # us -> 90 usec ( adding nop in inner loop -> 114 usec)
:m |us  ( n - )   
   7 #for  
      6 #  6 #for 6 #next  
   7 #next m;
: us  ( n - )   [ 7 push  6 push ]  |us  [ 6 pop  7 pop ] ;

:m |ms  ( n - )   
   7 #for 
      100 #  6 #for 
         81 #  5 #for 5 #next 
      6 #next 
   7 #next m;
: ms  ( n - )   [ 7 push  6 push  5 push ]  |ms  [ 5 pop  6 pop  7 pop ] ;

[then] \ ----------------------------------------------------------------------
\ 
\ -----[ LCD I/O ]
:m push-pull  $ff # P1MDOUT ior! m;  \ set all outputs as push-pull

:m pins  P1 m;
:m dirs  P1MDOUT m;

:m .e       [ 6 .P1 ] m;  
:m .rs      [ 7 .P1 ] m;    

-: instruction  [ .rs clr ] ;
-: data  [ .rs set ] ;

0 [if] \ ------------------[ backlight led ] ----------------------------------
:m bled     [ 5 .P2 ] m;
:m |+lcd      [ bled clr ] m;
: +lcd  |+lcd ;
:m |-lcd      [ bled set ] m;
: -lcd  |-lcd ;
: ~lcd      [ bled toggle ] ;
: flutter  10 # [ 7 push ] 7 #for  ~lcd  25 # ms 7 #next [ 7 pop ]  +lcd ;
[then] \ ----------------------------------------------------------------------
\ 
\ 160 us delay may be ok at half the value given
-: strobe   [ .e set ] 100 # us [ .e clr ]  160 # us  160 # us ;
\ 
\ -----[ lcd words ] 
\ note: $c4 swaps nibbles in the accumulator in one cycle
-: lcd  ( c - )
    $b0 # pins and!	\ output high nibble first
\    dup $f0 # and [ $c4 ] ,  pins ior!
    dup $f3 # and [ $c4 ] ,  pins ior!
    strobe
    $b0 # pins and!	\ now output do low nibble
\    $0f # and pins ior!
    $3f # and pins ior!
    strobe ;

-: lcd-command   lcd data 2 # ms ;

-: 0lcd       instruction   1 # lcd-command ;  \ reset lcd
-: lcd-home   instruction   2 # lcd-command ;  \ home cursor
-: lcd-off    instruction   8 # lcd-command ;
-: lcd-blank  instruction $0B # lcd-command ;
-: lcd-coff   instruction $0C # lcd-command ;  \ display on, cursor off
-: lcd-don    lcd-coff ;                       \ display on
-: lcd-blink  instruction $0D # lcd-command ;  \ blinking char at cursor
-: lcd-con    instruction $0E # lcd-command ;  \ cursor on
-: lcd-dblink instruction $0F # lcd-command ;  \ display, cursor, blink on
-: lcd-con    instruction $0E # lcd-command ;  \ cursor on
-: lcd-cleft  instruction $10 # lcd-command ;  \ move cursor left
-: lcd-cright instruction $14 # lcd-command ;  \ move cursor right
-: lcd-left   instruction $18 # lcd-command ;  \ scroll display one pos left
-: lcd-right  instruction $1C # lcd-command ;  \ scroll display one pos right
\ -: lcd-line1  instruction $80 # lcd-command ;  \ line1
\ -: lcd-line2  instruction $C0 # lcd-command ;  \ line2
-: line1  instruction $80 # lcd-command ;
-: line2  instruction $C0 # lcd-command ;

-: home&blink   lcd-home lcd-blink ;
\ 
\ 
\ --- send blank(s) to display
\ : _lcd  string "  "
-: _lcd  $20 # lcd ;
-: n_lcd  ( n -)   [ 4 push ] 4 #for _lcd 4 #next [ 4 pop ] ; \ n blanks
\
-: /line1   line1 16 # n_lcd line1 ;
-: /line2   line2 16 # n_lcd line2 ;

-: /lcd 
\    push-pull          \ set up in chip initialization
\    instruction
    $30 # pins #!	\ RS=0, instruction mode, 8-bit, 1/8 duty, 5X7
    $cf # dirs ior!	\ configure pins as outputs
    30 # ms		\ power on delay
\    $03 # pins #!	\ initialization pattern
    $33 # pins #!	\ initialization pattern
    strobe 10 # ms	
    strobe
    strobe
\    $02 # pins #!
    $32 # pins #!
    strobe 10 # ms
    $28 # lcd 10 # ms
    $0e # lcd
    $01 # lcd 10 # ms
    $02 # lcd
    data  10 # ms  ;

\ -----[ LCD Strings ]
\ 
\ changed to avoid conflicts with serial strings -- 04Apr rjn
\
\ : @p |@p ;     \ in standalone
\ : @p+ |@p+ ;   \ in standalone

:m lcd-type ( da) p! |@p+ begin |@p+ lcd 1- 0=until drop m;
\ def. below is in serial-string.fs or utils.fs
:m " 34 parse here there place here there [ c@ 1 + ] allot m;
-: lstring pop pop swap lcd-type ;

\ -: "zero  lstring " 0"

\ --- sample string
\ : greet string " It's MyForth!"
\ Example: /chip greet
\ 
\ -----[ HEX Display ]
\ : (digit)   -10 # + -if -39 # + then 97 # + ;  \ in debug.fs
-: lcd.digit   (digit) lcd ;
-: lcd-sign -if negate 45 # lcd ; then ;
\ 
\ 
0 [if] \------------------[ numeric display ]----------------------------------
\ 

\ : u/mod |u/mod ;
\ Avoid leading zeroes in (u.).
\ : three digit
\ : two digit digit ;
\ : (u.) 10 # u/mod 10 # u/mod if three ; then drop if two ; then drop
\   digit ;

-: u.lcd   10 # u/mod  10 # u/mod  lcd.digit  lcd.digit  lcd.digit ;
-: h.lcd   16 # u/mod lcd.digit lcd.digit ;

-: (lcd.cells)   [ 4 push ] 4 #for @+ h.lcd 4 #next [ 4 pop ] ;
-: lcd.4h   4 #  (lcd.cells) ;
-: lcd.cells  ( adr n -)  swap a!  (lcd.cells) ;

\ defined in send-utils.fs
\ : 10000s 0 # begin push -10000 ## d+ -if 10000 ## d+ pop ; then pop 1+ again
\ : 1000s 0 # begin push -1000 ## d+ -if 1000 ## d+ pop ; then pop 1+ again
\ : 100s 0 # begin push -100 ## d+ -if 100 ## d+ pop ; then pop 1+ again
\ 
-: ud.lcd  
   10000s lcd.digit 1000s lcd.digit 100s lcd.digit drop 
   10 # u/mod lcd.digit lcd.digit ;

[then] \ ----------------------------------------------------------------------   
\ 
0 [if] \ ------------------[ display stack on LCD ]----------------------------
\ 
-: l'empty  lstring "  empty!"
-: .hexes  4 #for @+ _lcd h.lcd 4 #next ;
-: .hexes1  S #@  1 # + a!  .hexes ;
-: .hexes2  S #@  5 # + a!  .hexes ;
-: (do-line)   depth 5 # <if drop depth ; then drop 4 # ;
-: do-line   (do-line) .hexes1 ;
-: ?line2   depth 5 # <if drop ; then drop  line2
   depth 10 # <if drop depth 4 # negate + .hexes2 ; then drop 
                                      5 # .hexes2 ;

-: s.hdr   lcd-coff  depth h.lcd [ char > ] # lcd ;

-: s.lcd   0lcd s.hdr depth 0=if drop l'empty ; then drop  do-line  ?line2 ;

\ must precede with line1 or line2 for placement
-: ss.lcd  s.hdr depth 0=if drop l'empty ; then drop  do-line ;
\ 
[then] \ ----------------------------------------------------------------------   

\   line2  depth 15 # ( |u/mod) h.lcd _lcd h.lcd ( ?line2) ; 
  \ 
0 [if] \ -------------------------[ LCD timing ]------------------------------
\ 
-: '16-chars  lstring " 1234567890123456"
-: (0lcd-test)  line2 '16-chars ;
: 0lcd-test   begin ~clue (0lcd-test) ie-switch 0=until. ;  \ 15 ms
\
[then] \ ---------------------------------------------------------------------
\ 
\ 
here ( *stop) [ swap -  cr .( lcd16.fs           ) . .( bytes) ]
\ 
\ 
0 [if] ------------------------[ revisions ]-----------------------------------

Date	   By  Comment
=======	=== ==================================================================
14Feb14  rjn changes for 16-pin LCD (2x16)  -- version for 850 MINT and STROBE    
01May13  rjn corrected duplicate naming of lcd-right (now lcd-cright)
22Apr13  rjn timing info
10Apr13  rjn added commands for scrolling, line changes, etc.
29Mar13  rjn modified for swis project, 16-pin LCD, string is now lstring	
28Feb09  rjn added FYI delays from 25_delays.fs
21Jan09  rjn changed for 410 port assignments
12Jul08  rjn added blink, blank and other functions
03Jul08  rjn added hex display words
27Jun08  rjn works with warbler I/O definitions (e.g., /chip)
22Aug06  rjn/cws  nibble mode LCD driver for C8051F310
------------------------------------------------------------------------ [then]
