\ send-multiply.fs -- Multi-byte multiply by a constant -- 04Sep12 rjn
\ 
] here  ( *start)
\ 
\ 
0 [if] \ ----------------[ Russian Peasant Multipication ]---------------------
\
1. Multiplies a double quad number by a 4-byte constant using the Russian 
   Peasant algorithm (shifts and adds).
2. Uses the freq register for the larger factor and 
   the scale register for the smaller factor.  The resulting product
   is in the prod register.  
3. Note: "dq" means "double quad", a 64-bit register.
\
[then] \ ----------------------------------------------------------------------
\ 
\ -----[ quad register, scale ]
\  
cpuHERE constant scale  4 cpuALLOT  \ scale is MSB, scale 3 + is LSB

:m scale1  [ scale 1 + ] m;
:m scale2  [ scale 2 + ] m;
:m scale3  [ scale 3 + ] m;
\ 
-: 0scale  0 ##  0 ##  scale #  !quad ;
\
0 [if] \ ---------------------------------------------------------------------- 
-: scale2*'  \ multiply scale register by 2, keeping carry result
   clrc  scale3 #@ 2*' scale3 #!  scale2 #@ 2*' scale2 #!
         scale1 #@ 2*' scale1 #!  scale  #@ 2*' scale  #! ;
[then] \ ----------------------------------------------------------------------
\ 
\ -----[ prod -- double quad product register ]
\  
cpuHERE constant prod 8 cpuALLOT  \ prod is MSB, prod 7 + is LSB
\ 
:m prod1  [ prod 1 + ] m;
:m prod2  [ prod 2 + ] m;
:m prod3  [ prod 3 + ] m;
:m prod4  [ prod 4 + ] m;
:m prod5  [ prod 5 + ] m;
:m prod6  [ prod 6 + ] m;
:m prod7  [ prod 7 + ] m;
\ 
-: 0prod  0 ##  0 ##  0 ##  0 ##  prod # !dq ;
\ -: @prod<  prod # @quad< ;
\ 
\ load prod (test)
\ : lprod   $78 # $56 # $34 # $02 #  $67 # $45 # $23 # $01 #  !prod ;  
\ 
\ -----[ freq -- double quad factor "a" register ]
\ 
\ multiply result ends up here
\ 
cpuHERE constant freq  8 cpuALLOT  \ freq is MSB, freq 7 + is LSB
\ 
:m freq1  [ freq 1 + ] m;
:m freq2  [ freq 2 + ] m;
:m freq3  [ freq 3 + ] m;
:m freq4  [ freq 4 + ] m;
:m freq5  [ freq 5 + ] m;
:m freq6  [ freq 6 + ] m;
:m freq7  [ freq 7 + ] m;
\
-: 0freq  0 ##  0 ##  0 ##  0 ##  freq #  !dq ;
:m prod>freq  prod # 8 # @cells<  freq #  !dq m;  
\
\ : lfreq   $78 # $56 # $34 # $02 #  $67 # $45 # $23 # $01 #  !freq ; \ test 
\
-: freq2*'  \ multiply contents of freq register by 2, keep carry result
   clrc   freq7 #@  2*' freq7 #!  freq6 #@  2*' freq6 #!  
          freq5 #@  2*' freq5 #!  freq4 #@  2*' freq4 #!  
          freq3 #@  2*' freq3 #!  freq2 #@  2*' freq2 #!  
          freq1 #@  2*' freq1 #!  freq  #@  2*' freq  #! ;
\ 
\ -----[ Add, Subtract freq to/from prod ]
\ 
:m |prod+freq
	[ t push  clrc
         prod7 t mov  freq7 addc  t prod7 mov
         prod6 t mov  freq6 addc  t prod6 mov
        	prod5 t mov  freq5 addc  t prod5 mov
         prod4 t mov  freq4 addc  t prod4 mov
        	prod3 t mov  freq3 addc  t prod3 mov
         prod2 t mov  freq2 addc  t prod2 mov
         prod1 t mov  freq1 addc  t prod1 mov
        	prod  t mov  freq  addc  t prod  mov
     t pop ]
m;
\
-: prod+freq  |prod+freq ;  \ results in prod
\ 
:m |freq+prod
	[ t push  clrc
         freq7 t mov  prod7 addc  t freq7 mov
         freq6 t mov  prod6 addc  t freq6 mov
        	freq5 t mov  prod5 addc  t freq5 mov
         freq4 t mov  prod4 addc  t freq4 mov
        	freq3 t mov  prod3 addc  t freq3 mov
         freq2 t mov  prod2 addc  t freq2 mov
         freq1 t mov  prod1 addc  t freq1 mov
        	freq  t mov  prod  addc  t freq  mov
     t pop ]
m;
\ 
-: freq+prod  |freq+prod ;  \ results in freq
\ 
1 [if] \ ----------------------------------------------------------------------
\ 
:m |prod-freq
	[ t push  clrc
         prod7 t mov  freq7 subb  t prod7 mov
         prod6 t mov  freq6 subb  t prod6 mov
        	prod5 t mov  freq5 subb  t prod5 mov
         prod4 t mov  freq4 subb  t prod4 mov
        	prod3 t mov  freq3 subb  t prod3 mov
         prod2 t mov  freq2 subb  t prod2 mov
        	prod1 t mov  freq1 subb  t prod1 mov
        	prod  t mov  freq  subb  t prod  mov
     t pop ]
m;
\
\ : prod-freq  |prod-freq ;  \ results in prod

:m |freq-prod
	[ t push  clrc
         freq7 t mov  prod7 subb  t freq7 mov
         freq6 t mov  prod6 subb  t freq6 mov
        	freq5 t mov  prod5 subb  t freq5 mov
         freq4 t mov  prod4 subb  t freq4 mov
        	freq3 t mov  prod3 subb  t freq3 mov
         freq2 t mov  prod2 subb  t freq2 mov
        	freq1 t mov  prod1 subb  t freq1 mov
        	freq  t mov  prod  subb  t freq  mov
     t pop ]
m;
\
-: freq-prod  |freq-prod ;  \ results in freq
\ 
[then] \ ----------------------------------------------------------------------
\ 
\ : cs$  string " cs! "  \ debug
\ 
-: scale2/' \ divide scale register by 2, accumulate remainder on carry result
   clrc  scale  #@  2/'  scale  #!  scale1 #@  2/' scale1 #!
         scale2 #@  2/'  scale2 #!  scale3 #@  2/' scale3 #!
   if' prod+freq ( can't be a macro!) ( cr cs$) ; then ;

-: scale2*' \ multiply scale register by 2
   clrc  scale3  #@  2*'  scale3  #!  scale2 #@  2*' scale2 #!
         scale1  #@  2*'  scale1  #!  scale  #@  2*' scale  #! ;
\ 
\ -----[ load scale ]
\ 
-: (msteps)  ( - n)  \ calculate multiply steps by looking for the last "1" 
   32 #  32 #  4 #for scale2*' if' ; then -1 # + 4 #next ;

\ assumes scale loaded, calculates number of multiply steps, preserves scale
-: msteps  ( - n)  scale # 4 # @cells<  (msteps)  push scale # 4 # !cells pop ;

\ note: 31 bytes, this is the last step: remainders are accumulated X 2
-: prod/2'  \ divide contents of prod register by 2, keep carry info   
   clrc   prod  #@  2/' prod  #!  prod1 #@  2/'  prod1 #!
          prod2 #@  2/' prod2 #!  prod3 #@  2/'  prod3 #!
          prod4 #@  2/' prod4 #!  prod5 #@  2/'  prod5 #!
          prod6 #@  2/' prod6 #!  prod7 #@  2/'  prod7 #! ;
\ 
1 [if] \ ---------------------------- prod ------------------------------------
\
-: multiply  \ multiply freq by scale, results in prod   
   0prod msteps 4 #for freq2*' scale2/' 4 #next  prod/2'  ;
\ 
\ load multiplicand in freq, results in prod
:m 10>scale  0scale $0a # scale3 #! m;
-: freq*10  10>scale  multiply  prod>freq ;
\
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ ----------------------------- test -----------------------------------
\
-: s$  string " start: "
-: f$  string " finish: "
-: .minfo   .scale .freq .prod ; \ display mult. regs
-: d$  string " -----------" 
\ 
\ load scale and freq first
-: multiply  \ multiply scale and freq, results in prod 
   0prod cr s$ .minfo
   msteps  4 #for cr d$  freq2*' scale2/'  .minfo  4 #next 
   prod/2' cr cr f$ .minfo ;
\
[then] \ ----------------------------------------------------------------------
\
0 [if] \ -------------------------- test --------------------------------------
\ 
\ Example for 50 MHz frequency, 125 MHz xtal, scale scaled by 2**22:
\ ($2FAF080)($89705F4)=$199999995FBA00
\ dividing by 2**22=$66666665=1,717,986,917 (vs. 1,717,986,918)
\
[then] \ ----------------------------------------------------------------------
\ 
\ 
here ( *stop)  [ swap -  cr .( multiply.fs        ) . .( bytes) ]
\ 
0 [if] \ ------------------------[ revisions ]---------------------------------
\ 
Date	By  Comment
=======	=== ===================================================================
04Sep12 rjn eliminated defs to reduce code size
30Aug12 rjn changed quadb name to scale (looks better on report)
14Aug12 rjn scaling by 2**22 yields a result within 1 Hertz
13Aug12 rjn now works for a double quad product
11Aug12 rjn initial version cloned from Oudin multiply
\ 
[then] \ ----------------------------------------------------------------------
