\ updates.fs -- frequency and LCD updates -- 18Mar14 rjn
\ 
]  here ( *start)
\
-: hold>ie   @hold<  ie-acc # !quad ;
-: ie>hold  ie-acc #  -int @quad< +int !hold ;
\ 
\ 
-: 'hertz  lstring "  Hertz" 
-: 'rpm    lstring "  rpm"

-: ?.units   @bx3  $01 # and
          0=if drop 'hertz ; then
   |1xor  0=if drop 'rpm   ; then  drop ;

-: update-lcd  ie>hold  line1  
   @bx3  $01 # and 
          0=if drop ?lcd.freq ?.units ; then 
   |1xor  0=if drop ?lcd.rpm  ?.units ; then drop ;  
\ 
\ note: it takes ~ 15 ms to position to a line and write 16 chars
\ -: ?update-lcd  @ticks0 -if update-lcd  25 # !ticks0 then drop ;
\ 
\ (A) takes 328 us to execute, up-test about the same
\ -: update-freq  ie>hold  (A) ;
\ : up-test   begin ~clue update-freq bswitch 0=until. ;

-: hold/2   \ divide hold by 2 -- change for PLL scheme -- 27May13 rjn
   clrc  hold0 #@  2/'  hold0 #!  hold1 #@  2/' hold1 #!
         hold2 #@  2/'  hold2 #!  hold3 #@  2/' hold3 #! ;

\ changed back for x1 scheme -- 27Sep13 rjn
-: update-freq  @hold< ( hold/2) (A) !hold ;

0 [if] \ -------------------[ frequency/LCD updates ]--------------------------
\ 
1. Because an LCD update takes around 20 ms, the application only checks for 
   LCD/frequency updates at a rate of approximately 25 ms.
2. To avoid redundant writes, the update algorithm also checks the contents of
   the hold register against the value captured at the last update.  If there
   is no change, there is no LCD or frequency update.
3. Only the two least-significant bytes of the hold register are checked
   because the encoder can't change more than about 25 counts in the 25 ms
   update interval (ie is updated every 1 ms).
\ 
[then] \ ----------------------------------------------------------------------

cpuHERE constant holder3  1 cpuALLOT  \ last hold3 (lsb) save
cpuHERE constant holder2  1 cpuALLOT  \ last hold2 (lsb+1) save

:m hchange?   hold3 #@  holder3 #@ xor  hold2 #@  holder2 #@  xor  ior  m;  
:m !holder    hold3 #@  holder3 #!  hold2 #@  holder2 #! m; 

-: do-updates  update-lcd  update-freq ;

-: ?do-updates  \ update only if hold has changed from last ie>hold xfr
   ie>hold  hchange? 0=if drop ; then drop !holder  do-updates  ;
\ check for updates ~ every 25 ms, only update if the ie-acc has changed
-: ?update   @ticks0 -if ?do-updates  25 # !ticks0 then drop ;
\ 
0 [if] \ ---------- test
\
\ See how much hold and holder differ when cranking fast clockwise
\ When really cranking hard, saw values as high as $18 (24), showing
\ that counts near/at the maximum can happen.  Perhaps the ISR needs 
\ to be shortened.  If so, the frequency updates may cause misses at a 
\ high rate (e.g., a 328 us frequency update and an ISR of 500 us).  
\ Hmmm
\ 
:m |hold-holder  \ result in holders
   [ t push  clrc
      hold3 t mov  holder3 subb  t holder3 mov
      hold2 t mov  holder2 subb  t holder2 mov
   t pop ] m;  

-: ?hold-change  ie>hold  hchange? 0=if drop ; then drop
    |hold-holder  lcd-home  holder3 #@  h.lcd !holder ; 

\ [ cr cr .s cr cr ]

-: test-updates 
    0lcd  $00 # hold3 #!  $00 # hold2 #!
    begin  
       @ticks0 -if ?hold-change  25 # !ticks0 then drop 
    bswitch 0=until. ;

[then] \ ---------------------
\ 
here ( *stop) [ swap -  ( --n)
cs? [if] cr .( updates.fs         ) . .( bytes) [else] drop [then] 
]
\ 
\  
0 [if] \ ------------------------[ revisions ]---------------------------------
\ 
Date	   By  Comment
=======	=== ===================================================================
18Mar14  rjn duty cycle changes
14Feb14  rjn slight changes (no test code) for dstro
27Sep13  rjn
28May13  rjn changed update algorithm -- now checks for change in hold since
              the last update
27May13  rjn changed for output/2, consolidated freq and LCD updates
30Apr13  rjn initial version
\ 
[then] \ ----------------------------------------------------------------------
