\ xvars.fs -- xram variables with flash save/restore -- 16Mar14 rjn
\ 
] here  ( *start)
\ 
\ 
\ 
0 [if] \ ----------------[ config & freq tables ]-----------------------------
\  
1. Saved variables are stored in flash and copied to xram at startup.
2. Values are stored contiguously in xram and the configuration block.
\ 
[then] \ ----------------------------------------------------------------------
\ 
0 constant xtable  \ start of ram table (mapped to loc. 0)

\ indexed is multiplied by 4 to yield quad values
-: >xtable  ( n - d)  push xtable ## pop ai>qadr ; \ index > xtable adr

\ --- given index, @ quad from xram table (i= 0 to 64)
\ table items should be stored msb first, like direct cell variables
-: @xtable   ( n - q)     >xtable @qx<  ;  \ msb on tos ( this is normal)
-: @xtable<  ( n - q< )   >xtable @qx   ;  \ lsb on tos ( e.g., ready to send)

\ --- given index, ! quad to xram table (i= 0 to 64)
-: !xtable  ( q n -)  >xtable !qx ;
\ 
\ indexed byte fetch and strore fro xtable: index is 0 to $ff
-: @bxtable   ( i - n)   0 # xtable ##  |d+ @xram ;  \ indexed byte @ from xram
-: !bxtable   ( n i -)   0 # xtable ##  |d+ !xram ;  \ indexed byte ! to xram
\ 
[
: extract-bytes  ( n n1 n2)  dup $00ff and  swap 8 0 do 2/ loop $00ff and ; 
: freq>ftable  ( n - lsb .. msb )  >r extract-bytes  r> extract-bytes ; 
]
\ 
1 [if] \ ---------------------[ configuration parameters ]---------------------
\ 
[ config-block ] constant ctable  \ config table starts at $1c00
here ( *) \ save current compilation pointer
ctable org
here ( adr at start of configuration block)
\ 8 gp byte values for @bx, !bx, etc.
$00 , ( main choice)      $00 , ( rate choice)  
$00 , ( units choice)     $00 , ( Units=Hertz)
$00 , ( rate=0.01 Hertz)  $00 , ( duty) 
$00 ,                     $00 ,
here constant end-ctable
\ 
\ calculate actual number of items in configuration table
end-ctable ( in config-block) [ swap - 2/ 2/ ] constant #ctable

-: ctable-ends ;  \ marker -- to see values, start looking backward from here
org ( *) \ finished compiling in configuration block 
-: >ctable  ( n - d)  push ctable ## pop ai>qadr ; \ index > flash table adr
\ 
\ --- given index, @ quad from xram table (i= 0 to 64)
-: @ctable   ( n - q)     >ctable @qflash<  ;  \ msb on tos, normal order
-: @ctable<  ( n - q< )   >ctable @qflash   ;  \ lsb on tos ready to send
\ 
\  
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ ---------------[ vfo frequency table ]--------------------------------
\ 
1. Normally, the vfo start frequency in Hertz is loaded from the table 
   and put into the incremental encoder register.  The DDS load value
   is calculated and put into the afreq register.
1. ftable entries are stored in "normal' MSB first order
2. Thus, @ftable retrieves value and puts MSB on tos
3. To put bytes on stack in "send" order, use @ftable<
4. For example, to output the second frequency to the DDS:
      1 # @ftable< /smode send 
5. ftable is stored just after ctable
\ 
[then] \ ----------------------------------------------------------------------
\ 
\ 
here ( *)
end-ctable constant ftable  \ starts just after ctable
ftable org ( compile in config-block)
here ( in config-block)
\ 
\ 
0 [if] \ ---------------[ ftable containing DDS load values ]------------------
\ 
\ 
[ 1000 hertz freq>ftable ] , , , ,  \ $00 , $00 , $86 , $37 ,
[ 2000 hertz freq>ftable ] , , , ,
[ 2100 hertz freq>ftable ] , , , ,
[ 3000 hertz freq>ftable ] , , , ,  \ $00 , $01 , $92 , $a7 ,
\ 
[then] \ ----------------------------------------------------------------------
\ 
1 [if] \ -------------[ ftable containing frequencies ]------------------------
\ 
[ 30000 0 freq>ftable ] , , , ,  \ 300.00 Hertz
[  2000 0 freq>ftable ] , , , ,  \ 20.00 Hertz
[  3000 0 freq>ftable ] , , , ,  \ 30.00 Hertz
\ 
[then] \ ----------------------------------------------------------------------
\  
\ calculate actual number of items in frequency table (e.g., for clip, display)
here constant end-ftable
end-ftable [ swap - 2/ 2/ ] constant #ftable
-: ftable-ends ;  \ marker -- to see values, start looking backward from here
org ( *) \ finished compiling in configuration block  
\ 
-: >ftable  ( n - d)  push ftable ## pop ai>qadr ; \ index > flash table adr
\ 
\ --- given index, @ quad from flash table (i= 0 to 64)
-: @ftable   ( n - q)     >ftable @qflash<  ;  \ msb on tos, normal order
-: @ftable<  ( n - q< )   >ftable @qflash   ;  \ lsb on tos ready to send
\ 
-: @fxtable  ( i - q)   #ctable # + @xtable ;
-: !fxtable  ( q i -)   #ctable # + !xtable ;

\ 
\ note: now restores in msb first order!
-: restore-xtable  \ copy ftable in flash to xram freq. table
   0 # gp8 #!  \ gp8 holds current ftable index
   [ 7 push ]  #ftable #  #ctable # +  7 #for
      ( ftable ##) gp8 #@ ( ai>qadr @qflash<) @ctable ( - q) 
      gp8 #@  !xtable 
      [ gp8 inc ] \ point to next quad
   7 #next  [ 7 pop ] ;
\ 
\ must erase flash first with erase-config
-: save-xtable  \ save xram freqs to flash
   0 # gp8 #!  \ gp8 holds current ftable index
   [ 7 push ]  #ftable #  #ctable # +  ( # quads) 7 #for
      gp8 #@  @xtable
      ctable ## gp8 #@ ai>qadr p! 
\      push push push !f+ pop !f+ pop !f+ pop !f+  
      !f+ !f+ !f+ !f+
      [ gp8 inc ] \ point to next quad
   7 #next  [ 7 pop ] ;  

-: xfreq>hold  ( -)   0 # @xtable  !hold ;
\ 
-: hold>xfreq  ( -)   @hold<  0 # !xtable ;
\ 
1 [if] \ ------------------[ xram variables ]----------------------------------
\ 
: @bx0  ( - n)     0 # @bxtable ;
: @bx1  ( - n)     1 # @bxtable ;  
: @bx2  ( - n)     2 # @bxtable ;  
: @bx3  ( - n)     3 # @bxtable ;  
: @bx4  ( - n)     4 # @bxtable ;
: @bx5  ( - n)     5 # @bxtable ;  
: @bx6  ( - n)     6 # @bxtable ;  
: @bx7  ( - n)     7 # @bxtable ;  
\ 
: !bx0  ( n -)     0 # !bxtable ;
: !bx1  ( n -)     1 # !bxtable ;  
: !bx2  ( n -)     2 # !bxtable ;  
: !bx3  ( n -)     3 # !bxtable ;  
: !bx4  ( n -)     4 # !bxtable ;
: !bx5  ( n -)     5 # !bxtable ;  
: !bx6  ( n -)     6 # !bxtable ;  
: !bx7  ( n -)     7 # !bxtable ;   
\ 
[then] \ ----------------------------------------------------------------------
\ 
0 [if] \ -----[ debug, test ]
\ 
: .ftable  \ display config page entries for frequency table
   0 # gp8 #!  \ gp8 holds current ftable index
   [ 7 push ]  #ftable #  7 #for  \ show only active freqs
      ftable ## gp8 #@  ai>qadr 2dup cr dh. space  @qflash 
      ( cr gp8 #@  h. space)  push push push h. pop h. pop h. pop h.
      [ gp8 inc ]
   7 #next  [ 7 pop ] cr ;   

: .xtable  -1 #
   #ftable #  6 #for 
      cr 1+ dup dup h. space
      @xtable push push push h. pop h. pop h. pop h.  
   6 #next drop cr ;

: fxtest  \ load some test data into upper part of table
\   lsb ---------------- msb  idx
   $03 #  $02 #  $01 #  $00 # 0 # !xtable
   $07 #  $06 #  $05 #  $04 # 1 # !xtable
   $0b #  $0a #  $09 #  $08 # 2 # !xtable
   $0f #  $0e #  $0d #  $0c # 3 # !xtable
   $13 #  $12 #  $11 #  $10 # 4 # !xtable
   $17 #  $16 #  $15 #  $14 # 5 # !xtable
   $1b #  $1a #  $19 #  $18 # 6 # !xtable
   $1f #  $1e #  $1d #  $1c # 7 # !xtable ; 
\ 
[then] \ ----------------------------------------------------------------------  
\ 
here ( *stop) [ swap -  cr .( xvars.fs           ) . .( bytes) ]
\ 
0 [if] \ ------------------------[ revisions ]--------------------------------- 

Date	  By  Comment
======= === ===================================================================
16Mar14 rjn moved start of config block to $1e00 from $1c00
02Mar14 rjn renamed from fxtable.fs, new "generic" scheme
24Feb14 rjn reworked table for DDS Strobe
23Apr13 rjn moved ~send to s-regs.fs, renamed from send-fxtable.fs
21Apr13 rjn initial verson
\ 
[then] \ ----------------------------------------------------------------------
